SystemOrganization addCategory: #XPacMan!

Object subclass: #GameElement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPacMan'!

!GameElement class methodsFor: 'as yet unclassified' stamp: 'mw 6/9/2012 14:52'!
elementExtent
	"returns the with and height of every game element"
	^30@30! !

!GameElement methodsFor: 'drawing' stamp: 'mw 6/17/2012 19:00'!
calcPosition: loc
	| delta x1 x2 y1 y2 |
	delta := GameElement elementExtent.
	x1 := (loc x - 1) * delta x.
	x2 := x1 + delta x.
	y1 := (loc y - 1) * delta y.
	y2 := y1 + delta y.
	^Rectangle left: x1 right: x2 top: y1 bottom:y2! !

!GameElement methodsFor: 'logic' stamp: 'tm 6/26/2012 00:46'!
canBeMovedOn
	^ true! !

!GameElement methodsFor: 'drawing' stamp: 'mw 6/30/2012 23:28'!
hasPoints
	"default implementation"
	^false! !

!GameElement methodsFor: 'drawing' stamp: 'mw 6/17/2012 20:13'!
render: form at: loc
	"should render the element to its position on the screen"
	| line line2 line3 line4 position rect |
	position := self calcPosition: loc.
	rect := Line
		from: (position left)@(position top)
		to: (position right)@(position top)
		withForm: (Form extent: 1@(GameElement elementExtent y) depth: 8) fillWhite.
	rect displayOn: form.
	line := Line
		from: (position left)@(position top)
		to: (position right)@(position top)
		withForm: (Form extent: 1@1 depth: 8) fillBlack.
	line displayOn: form.
	line2 := Line
		from: (position right)@(position top)
		to: (position right)@(position bottom)
		withForm: (Form extent: 1@1 depth: 8) fillBlack.
	line2 displayOn: form.
	line3 := Line
		from: (position right)@(position bottom)
		to: (position left)@(position bottom)
		withForm: (Form extent: 1@1 depth: 8) fillBlack.
	line3 displayOn: form.
	line4 := Line
		from: (position left)@(position bottom)
		to: (position left)@(position top)
		withForm: (Form extent: 1@1 depth: 8) fillBlack.
	line4 displayOn: form! !

GameElement subclass: #Pawn
	instanceVariableNames: 'playingField color position direction points isDead'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPacMan'!

!Pawn class methodsFor: 'directions' stamp: 'tm 6/26/2012 00:19'!
directionDown
	"Returns constant direction value."
	
	^ 'down'! !

!Pawn class methodsFor: 'directions' stamp: 'tm 6/26/2012 00:19'!
directionLeft
	"Returns constant direction value."
	
	^ 'left'! !

!Pawn class methodsFor: 'directions' stamp: 'tm 6/26/2012 00:19'!
directionRight
	"Returns constant direction value."
	
	^ 'right'! !

!Pawn class methodsFor: 'directions' stamp: 'tm 6/26/2012 00:20'!
directionUp
	"Returns constant direction value."
	
	^ 'up'! !

!Pawn methodsFor: 'accessing' stamp: 'mw 7/3/2012 23:11'!
at: otherPos
	^(otherPos = position)! !

!Pawn methodsFor: 'moving' stamp: 'tm 6/26/2012 01:00'!
canMoveTo: newPosition
	| intercepting canBeMovedOn |

	intercepting := self playingField at: newPosition.
	canBeMovedOn := intercepting canBeMovedOn.
	
	^ canBeMovedOn! !

!Pawn methodsFor: 'drawing' stamp: 'fdskhu 7/4/2012 22:22'!
changeColor
   | oldColor |
	
	oldColor := self color.
	
		(oldColor = Color blue) ifTrue: [			
			Transcript show: self color; show: 'changes to green';cr.
			self color: Color green.
		].
	
		(oldColor = Color red) ifTrue: [
			Transcript show: self color; show: 'changes to blue';cr.
			self color: Color blue.
		].
	
		(oldColor = Color green) ifTrue: [
			Transcript show: self color; show: 'changes to red';cr.
			self color: Color red.
		].
		
		
! !

!Pawn methodsFor: 'accessing' stamp: 'mw 6/9/2012 16:13'!
color
	"Answer the value of color"

	^ color! !

!Pawn methodsFor: 'accessing' stamp: 'tm 6/25/2012 23:57'!
color: anObject
	"Set the value of direction"

	color := anObject! !

!Pawn methodsFor: 'logic' stamp: 'mw 7/4/2012 00:08'!
consumes: otherPlayer
	"hier anderen spieler fressen"
	Transcript show: color; show: ' eats '; show: otherPlayer color; cr.
	
	self points: (self points + otherPlayer points).
	otherPlayer points: 0.
	otherPlayer isDead: true.! !

!Pawn methodsFor: 'accessing' stamp: 'tm 6/25/2012 23:57'!
direction
	"Answer the value of direction"

	^ direction! !

!Pawn methodsFor: 'accessing' stamp: 'tm 6/26/2012 01:13'!
direction: aDirection
	"Set the value of direction"
	
	direction ~= aDirection ifTrue: [
		Transcript show: 'changing direction to ', aDirection; cr.
	].

	direction := aDirection! !

!Pawn methodsFor: 'accessing' stamp: 'mw 7/4/2012 00:07'!
isDead
	isDead isNil ifTrue: [self isDead: false].
	^isDead! !

!Pawn methodsFor: 'accessing' stamp: 'mw 7/4/2012 00:06'!
isDead: anObject
	"Set the value of isDead"

	isDead := anObject! !

!Pawn methodsFor: 'moving' stamp: 'mw 7/3/2012 22:56'!
move: direction
	| newpos canMove |
	(direction = Pawn directionDown) ifTrue: [
		newpos := (position x)@(position y + 1).
	].
	(direction = Pawn directionUp) ifTrue: [
		newpos := (position x)@(position y - 1).
	].
	(direction = Pawn directionLeft) ifTrue: [
		newpos := (position x - 1)@(position y ).
	].
	(direction = Pawn directionRight) ifTrue: [
		newpos := (position x + 1)@(position y).
	].
	newpos isNil ifFalse: [
		canMove := self canMoveTo: newpos.
		canMove ifTrue: [
			self tryConsume: newpos.
			self position: newpos.
		].
		canMove ifFalse: [
			Transcript show: 'cannot move in direction ', direction; cr.
		].
	].! !

!Pawn methodsFor: 'accessing' stamp: 'tm 6/26/2012 00:49'!
playingField
	"Answer the value of playingField"

	^ playingField! !

!Pawn methodsFor: 'accessing' stamp: 'tm 6/26/2012 00:49'!
playingField: aPlayingField
	"Set the value of playingField"

	playingField := aPlayingField! !

!Pawn methodsFor: 'accessing' stamp: 'mw 6/29/2012 23:48'!
points
	"Answer the value of points"

	^ points! !

!Pawn methodsFor: 'accessing' stamp: 'mw 6/29/2012 23:48'!
points: anObject
	"Set the value of points"

	points := anObject! !

!Pawn methodsFor: 'accessing' stamp: 'mw 6/17/2012 18:53'!
position
	"Answer the value of position"

	^ position! !

!Pawn methodsFor: 'accessing' stamp: 'tm 6/25/2012 23:47'!
position: anObject
	"Set the value of position"

	position := anObject! !

!Pawn methodsFor: 'drawing' stamp: 'mw 7/4/2012 00:16'!
render: form at: loc
	| pos fillForm circle offset radius |
	offset := GameElement elementExtent.
	radius := (offset x/2) - 2.
	pos := self calcPosition: loc.
	circle := Circle new.
	fillForm := Form extent: 2@2 depth: 8.
	fillForm fillColor: self color.
	circle form: fillForm.
	circle radius: radius.
	circle center: (pos left + (offset x / 2))@(pos top + (offset y / 2)).
	circle displayOn: form.
	
	self isDead
		ifTrue: [
			| rect |
			rect := Line
				from: (pos left)@(pos top)
				to: (pos right)@(pos bottom)
				withForm: fillForm.
				
			rect displayOn: form.
		]! !

!Pawn methodsFor: 'logic' stamp: 'mw 7/3/2012 23:20'!
tryConsume: newPosition
	| intercepting isPawn otherPlayer |

	intercepting := self playingField at: newPosition.
	isPawn := false.
	
	(self playingField pawn1 at: newPosition) ifTrue: [isPawn := true. otherPlayer := self playingField pawn1].
	(self playingField pawn2 at: newPosition) ifTrue: [isPawn := true. otherPlayer := self playingField pawn2].
	(self playingField pawn3 at: newPosition)
		ifTrue: [
			Transcript show: 'player 3 is on this field'; cr.
			isPawn := true. 
			otherPlayer := self playingField pawn3.
		].
	
	isPawn
		ifTrue: [
			Transcript show: 'checking intercepting pawn to eat'; cr.
			(self color = Color green) ifTrue: [
				(otherPlayer color = Color red) ifTrue: [
					self consumes: otherPlayer.
				] ifFalse: [ "can only be blue"
					otherPlayer consumes: self.
				]
			].
			(self color = Color red) ifTrue: [
				(otherPlayer color = Color blue) ifTrue: [
					self consumes: otherPlayer.
				] ifFalse: [ "can only be green"
					otherPlayer consumes: self.
				]
			].
			(self color = Color blue) ifTrue: [
				(otherPlayer color = Color green) ifTrue: [
					self consumes: otherPlayer.
				] ifFalse: [ "can only be red"
					otherPlayer consumes: self.
				]
			]
		]
		ifFalse: [
			intercepting hasPoints
				ifTrue: [
					Transcript show: 'eating points '.
					self points: (self points + intercepting points).
					intercepting points: 0.
				].
		]
! !

GameElement subclass: #PointField
	instanceVariableNames: 'points'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPacMan'!

!PointField methodsFor: 'drawing' stamp: 'mw 6/30/2012 23:33'!
hasPoints
	^self points > 0! !

!PointField methodsFor: 'accessing' stamp: 'mw 6/30/2012 23:31'!
points
	"Answer the value of points"
	points isNil ifTrue: [points := 1].
	^  points! !

!PointField methodsFor: 'accessing' stamp: 'mw 6/30/2012 00:20'!
points: anObject
	"Set the value of points"

	points := anObject! !

!PointField methodsFor: 'drawing' stamp: 'mw 6/30/2012 23:23'!
render: form at: loc
	super render: form at: loc.
	self hasPoints
		ifTrue: [
			| pos fillForm circle offset radius |
			offset := GameElement elementExtent.
			radius := (offset x/2) - 4.
			pos := self calcPosition: loc.
			circle := Circle new.
			fillForm := Form extent: 5@5 depth: 8.
			fillForm fillColor: Color yellow.
			circle form: fillForm.
			circle radius: radius.
			circle center: (pos left + (offset x / 2))@(pos top + (offset y / 2)).
			circle displayOn: form.
		].! !

GameElement subclass: #WallField
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPacMan'!

!WallField methodsFor: 'logic' stamp: 'tm 6/26/2012 00:47'!
canBeMovedOn
	^ false! !

!WallField methodsFor: 'drawing' stamp: 'mw 6/17/2012 19:01'!
render: form at: loc
	| line position |
	super render: form at: loc.
	position := self calcPosition: loc.
	line := Line
		from: (position left)@(position top)
		to: (position right)@(position bottom)
		withForm: (Form extent: 1@1 depth: 8) fillBlack.
	line displayOn: form! !

Object subclass: #PlayingField
	instanceVariableNames: 'fields numRows numCeils pawn1 pawn2 pawn3'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPacMan'!

!PlayingField class methodsFor: 'as yet unclassified' stamp: 'mw 6/9/2012 14:34'!
newOfSize: aPoint
	| model |
	model := self basicNew.
	model
		numRows: aPoint y;
		numCeils: aPoint x.
	model initialize.
	^model! !

!PlayingField methodsFor: 'accessing' stamp: 'mw 6/9/2012 13:56'!
at: aPoint
	^self fields at: aPoint! !

!PlayingField methodsFor: 'accessing' stamp: 'mw 6/9/2012 13:57'!
at: aPoint put: aField
	self fields at: aPoint put: aField! !

!PlayingField methodsFor: 'accessing' stamp: 'mw 6/3/2012 18:20'!
fields
	"Answer the value of fields"

	^ fields! !

!PlayingField methodsFor: 'accessing' stamp: 'mw 6/3/2012 18:20'!
fields: anObject
	"Set the value of fields"

	fields := anObject! !

!PlayingField methodsFor: 'initialize-release' stamp: 'mw 6/3/2012 18:20'!
initialize
	super initialize.
	self initializeFields.! !

!PlayingField methodsFor: 'initialize-release' stamp: 'mw 6/3/2012 18:23'!
initializeFields
	self fields: Dictionary new.
	1 to: self numCeils do: [:x |
		1 to: self numRows do: [:y |
			| pt field |
			pt := x@y.
			field := GameElement new.
			self fields at: pt put: field]]! !

!PlayingField methodsFor: 'accessing' stamp: 'mw 6/3/2012 18:24'!
numCeils
	"Answer the value of numCeils"
	numCeils isNil ifTrue: [self numCeils: 10].
	^ numCeils! !

!PlayingField methodsFor: 'accessing' stamp: 'mw 6/3/2012 18:20'!
numCeils: anObject
	"Set the value of numCeils"

	numCeils := anObject! !

!PlayingField methodsFor: 'accessing' stamp: 'mw 6/3/2012 18:25'!
numRows
	"Answer the value of numRows"
	numRows isNil ifTrue: [self numRows: 10].
	^ numRows! !

!PlayingField methodsFor: 'accessing' stamp: 'mw 6/3/2012 18:20'!
numRows: anObject
	"Set the value of numRows"

	numRows := anObject! !

!PlayingField methodsFor: 'accessing' stamp: 'mw 6/17/2012 18:47'!
pawn1
	"Answer the value of pawn1"

	^ pawn1! !

!PlayingField methodsFor: 'accessing' stamp: 'mw 6/17/2012 18:47'!
pawn1: anObject
	"Set the value of pawn1"

	pawn1 := anObject! !

!PlayingField methodsFor: 'accessing' stamp: 'mw 6/17/2012 18:47'!
pawn2
	"Answer the value of pawn2"

	^ pawn2! !

!PlayingField methodsFor: 'accessing' stamp: 'mw 6/17/2012 18:47'!
pawn2: anObject
	"Set the value of pawn2"

	pawn2 := anObject! !

!PlayingField methodsFor: 'accessing' stamp: 'mw 6/17/2012 18:47'!
pawn3
	"Answer the value of pawn3"

	^ pawn3! !

!PlayingField methodsFor: 'accessing' stamp: 'mw 6/17/2012 18:47'!
pawn3: anObject
	"Set the value of pawn3"

	pawn3 := anObject! !

!PlayingField methodsFor: 'drawing' stamp: 'mw 6/29/2012 23:36'!
render: form
	"should render the PlayingField and it child elements to the form"
	| elemLoc elem |
	1 to: numCeils do: [:y |
		1 to: numRows do: [:x |
			elemLoc := x@y.
			elem := self at: elemLoc.
			elem render: form at: elemLoc.
			(pawn1 at: elemLoc) ifTrue: [pawn1 render: form at: elemLoc.].
			(pawn2 at: elemLoc) ifTrue: [pawn2 render: form at: elemLoc.].
			(pawn3 at: elemLoc) ifTrue: [pawn3 render: form at: elemLoc.].
			]].! !

Morph subclass: #PacGame
	instanceVariableNames: 'playingField fieldForm running rounds colorProcess'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPacMan'!

!PacGame class methodsFor: 'as yet unclassified' stamp: 'fdskhu 7/4/2012 22:18'!
createPlayers: field from: pacGame
	| player1  player2 player3 thread |
	
	player1 := Pawn new.
	player1 color: Color green.
	player1 position: 2@2.
	"player1 direction: Pawn directionDown."
	player1 playingField: field.
	player1 points: 1.
	field pawn1: player1.
	
	player2 := Pawn new.
	player2 color: Color blue.
	player2 position: 8@2.
	"player2 direction: Pawn directionDown."
	player2 playingField: field.
	player2 points: 1.
	field pawn2: player2.
	
	player3 := Pawn new.
	player3 color: Color red.
	player3 position: 4@9.
	"player3 direction: Pawn directionDown."
	player3 playingField: field.
	player3 points: 1.
	field pawn3: player3.
	

	thread := [
	    [true] whileTrue: [
		(Delay forSeconds:30) wait.
		Transcript show: 'player1:'; cr.
		player1 changeColor.
		Transcript show: 'player2'; cr.
		player2 changeColor.
		Transcript show: 'player3'; cr.
		player3 changeColor.
		Transcript show: 'testend'; cr.
	    ]
	  ] fork.

	pacGame colorProcess: thread.! !

!PacGame class methodsFor: 'as yet unclassified' stamp: 'is 7/4/2012 10:38'!
createPlaying2Field
	| field |
	field := PlayingField newOfSize: 10@10.
	field at: 1@1 put: WallField new.
	field at: 2@1 put: WallField new.
	field at: 3@1 put: WallField new.
	field at: 4@1 put: WallField new.
	field at: 5@1 put: WallField new.
	field at: 6@1 put: WallField new.
	field at: 7@1 put: WallField new.
	field at: 8@1 put: WallField new.
	field at: 9@1 put: WallField new.
	field at: 10@1 put: WallField new.
	field at: 10@2 put: WallField new.
	field at: 10@3 put: WallField new.
	field at: 10@4 put: WallField new.
	field at: 10@5 put: WallField new.
	field at: 10@6 put: WallField new.
	field at: 10@7 put: WallField new.
	field at: 10@8 put: WallField new.
	field at: 10@9 put: WallField new.
	field at: 10@10 put: WallField new.
	field at: 1@1 put: WallField new.
	field at: 1@2 put: WallField new.
	field at: 1@3 put: WallField new.
	field at: 1@4 put: WallField new.
	field at: 1@5 put: WallField new.
	field at: 1@6 put: WallField new.
	field at: 1@7 put: WallField new.
	field at: 1@8 put: WallField new.
	field at: 1@9 put: WallField new.
	field at: 1@10 put: WallField new.
	field at: 2@10 put: WallField new.
	field at: 3@10 put: WallField new.
	field at: 4@10 put: WallField new.
	field at: 5@10 put: WallField new.
	field at: 6@10 put: WallField new.
	field at: 7@10 put: WallField new.
	field at: 8@10 put: WallField new.
	field at: 9@10 put: WallField new.
	field at: 10@10 put: WallField new.
	
	field at: 9@9 put: PointField new.
	field at: 8@9 put: PointField new.
	field at: 7@9 put: PointField new.
	field at: 6@9 put: PointField new.
	field at: 5@9 put: PointField new.
	field at: 3@9 put: PointField new.
	field at: 2@9 put: PointField new.

	field at: 9@8 put: PointField new.
	field at: 8@8 put: WallField new.
	field at: 7@8 put: WallField new.
	field at: 6@8 put: WallField new.
	field at: 5@8 put: PointField new.
	field at: 4@8 put: PointField new.
	field at: 3@8 put: PointField new.
	field at: 2@8 put: PointField new.

	field at: 9@7 put: PointField new.
	field at: 8@7 put: PointField new.
	field at: 7@7 put: PointField new.
	field at: 6@7 put: WallField new.
	field at: 5@7 put: PointField new.
	field at: 4@7 put: WallField new.
	field at: 3@7 put: WallField new.
	field at: 2@7 put: PointField new.
	
	field at: 9@6 put: PointField new.
	field at: 8@6 put: WallField new.
	field at: 7@6 put: PointField new.
	field at: 6@6 put: PointField new.
	field at: 5@6 put: WallField new.
	field at: 4@6 put: PointField new.
	field at: 3@6 put: PointField new.
	field at: 2@6 put: PointField new.
	
	field at: 9@5 put: PointField new.
	field at: 8@5 put: WallField new.
	field at: 7@5 put: PointField new.
	field at: 6@5 put: PointField new.
	field at: 5@5 put: WallField new.
	field at: 4@5 put: WallField new.
	field at: 3@5 put: WallField new.
	field at: 2@5 put: PointField new.
	
	field at: 9@4 put: PointField new.
	field at: 8@4 put: WallField new.
	field at: 7@4 put: WallField new.
	field at: 6@4 put: WallField new.
	field at: 5@4 put: PointField new.
	field at: 4@4 put: PointField new.
	field at: 3@4 put: WallField new.
	field at: 2@4 put: PointField new.
	
	field at: 9@3 put: PointField new.
	field at: 8@3 put: PointField new.
	field at: 7@3 put: PointField new.
	field at: 6@3 put: PointField new.
	field at: 5@3 put: PointField new.
	field at: 4@3 put: PointField new.
	field at: 3@3 put: PointField new.
	field at: 2@3 put: PointField new.
	
	field at: 9@2 put: PointField new.
	field at: 7@2 put: PointField new.
	field at: 6@2 put: WallField new.
	field at: 5@2 put: PointField new.
	field at: 4@2 put: PointField new.
	field at: 3@2 put: WallField new.
	
	
	^field! !

!PacGame class methodsFor: 'as yet unclassified' stamp: 'is 7/4/2012 10:34'!
createPlayingField
	| field |
	field := PlayingField newOfSize: 10@10.
	field at: 1@1 put: WallField new.
	field at: 2@1 put: WallField new.
	field at: 3@1 put: WallField new.
	field at: 4@1 put: WallField new.
	field at: 5@1 put: WallField new.
	field at: 6@1 put: WallField new.
	field at: 7@1 put: WallField new.
	field at: 8@1 put: WallField new.
	field at: 9@1 put: WallField new.
	field at: 10@1 put: WallField new.
	field at: 10@2 put: WallField new.
	field at: 10@3 put: WallField new.
	field at: 10@4 put: WallField new.
	field at: 10@5 put: WallField new.
	field at: 10@6 put: WallField new.
	field at: 10@7 put: WallField new.
	field at: 10@8 put: WallField new.
	field at: 10@9 put: WallField new.
	field at: 10@10 put: WallField new.
	field at: 1@1 put: WallField new.
	field at: 1@2 put: WallField new.
	field at: 1@3 put: WallField new.
	field at: 1@4 put: WallField new.
	field at: 1@5 put: WallField new.
	field at: 1@6 put: WallField new.
	field at: 1@7 put: WallField new.
	field at: 1@8 put: WallField new.
	field at: 1@9 put: WallField new.
	field at: 1@10 put: WallField new.
	field at: 2@10 put: WallField new.
	field at: 3@10 put: WallField new.
	field at: 4@10 put: WallField new.
	field at: 5@10 put: WallField new.
	field at: 6@10 put: WallField new.
	field at: 7@10 put: WallField new.
	field at: 8@10 put: WallField new.
	field at: 9@10 put: WallField new.
	field at: 10@10 put: WallField new.
	
	field at: 9@9 put: PointField new.
	field at: 8@9 put: PointField new.
	field at: 7@9 put: PointField new.
	field at: 6@9 put: PointField new.
	field at: 5@9 put: PointField new.
	field at: 3@9 put: PointField new.
	field at: 2@9 put: PointField new.

	field at: 9@8 put: WallField new.
	field at: 8@8 put: WallField new.
	field at: 7@8 put: PointField new.
	field at: 6@8 put: WallField new.
	field at: 5@8 put: PointField new.
	field at: 4@8 put: PointField new.
	field at: 3@8 put: WallField new.
	field at: 2@8 put: PointField new.

	field at: 9@7 put: PointField new.
	field at: 8@7 put: PointField new.
	field at: 7@7 put: PointField new.
	field at: 6@7 put: WallField new.
	field at: 5@7 put: PointField new.
	field at: 4@7 put: WallField new.
	field at: 3@7 put: WallField new.
	field at: 2@7 put: PointField new.
	
	field at: 9@6 put: PointField new.
	field at: 8@6 put: WallField new.
	field at: 7@6 put: PointField new.
	field at: 6@6 put: PointField new.
	field at: 5@6 put: PointField new.
	field at: 4@6 put: PointField new.
	field at: 3@6 put: PointField new.
	field at: 2@6 put: PointField new.
	
	field at: 9@5 put: PointField new.
	field at: 8@5 put: WallField new.
	field at: 7@5 put: PointField new.
	field at: 6@5 put: PointField new.
	field at: 5@5 put: PointField new.
	field at: 4@5 put: PointField new.
	field at: 3@5 put: WallField new.
	field at: 2@5 put: WallField new.
	
	field at: 9@4 put: PointField new.
	field at: 8@4 put: WallField new.
	field at: 7@4 put: WallField new.
	field at: 6@4 put: WallField new.
	field at: 5@4 put: PointField new.
	field at: 4@4 put: PointField new.
	field at: 3@4 put: WallField new.
	field at: 2@4 put: PointField new.
	
	field at: 9@3 put: PointField new.
	field at: 8@3 put: PointField new.
	field at: 7@3 put: PointField new.
	field at: 6@3 put: PointField new.
	field at: 5@3 put: PointField new.
	field at: 4@3 put: PointField new.
	field at: 3@3 put: PointField new.
	field at: 2@3 put: PointField new.
	
	field at: 9@2 put: PointField new.
	field at: 7@2 put: PointField new.
	field at: 6@2 put: WallField new.
	field at: 5@2 put: PointField new.
	field at: 4@2 put: PointField new.
	field at: 3@2 put: WallField new.
	
	
	^field! !

!PacGame class methodsFor: 'as yet unclassified' stamp: 'mw 6/9/2012 14:57'!
fieldExtendFor: aPlayingField
	"calculates the required size for a form containint aPlayingField"
	| x y |
	x := GameElement elementExtent x * aPlayingField numCeils.
	y := GameElement elementExtent y * aPlayingField numRows.
	^x@y! !

!PacGame class methodsFor: 'as yet unclassified' stamp: 'mw 7/4/2012 00:18'!
resetPlayers: field from: oldField
	field pawn1: oldField pawn1.
	field pawn1 position: 2@2.
	field pawn1 playingField: field.
	field pawn1 isDead: false.
	field pawn1 direction: nil.

	field pawn2: oldField pawn2.
	field pawn2 position: 8@2.
	field pawn2 playingField: field.
	field pawn2 isDead: false.
	field pawn2 direction: nil.

	field pawn3: oldField pawn3.
	field pawn3 position: 4@9.
	field pawn3 playingField: field.
	field pawn3 isDead: false.
	field pawn3 direction: nil.
! !

!PacGame methodsFor: 'drawing' stamp: 'mw 6/9/2012 17:44'!
calculateExtent
	| pt |
	pt := self fieldForm extent.
	pt := pt + (self panelWidth@0).
	^pt! !

!PacGame methodsFor: 'accessing' stamp: 'd 7/4/2012 22:04'!
colorProcess: anObject
	"Set the value of colorProcess"

	colorProcess := anObject! !

!PacGame methodsFor: 'accessing' stamp: 'mw 6/9/2012 17:40'!
fieldForm
	"Answer the value of fieldForm"

	^ fieldForm! !

!PacGame methodsFor: 'accessing' stamp: 'mw 6/9/2012 17:40'!
fieldForm: anObject
	"Set the value of fieldForm"

	fieldForm := anObject! !

!PacGame methodsFor: 'drawing' stamp: 'mw 6/30/2012 00:07'!
findPlayerCounter: player
    ^self allMorphs detect: [:m | m knownName = player] ifNone: []! !

!PacGame methodsFor: 'keyboard' stamp: 'mw 6/13/2012 20:05'!
handlesKeyboard: evt
	^true! !

!PacGame methodsFor: 'initialize-release' stamp: 'd 7/4/2012 22:03'!
initialize
	Transcript show: 'Initializing Morph'; cr. 
	super initialize.
	self playingField: PacGame createPlayingField.
	
	PacGame createPlayers: self playingField from: self.
	self fieldForm: (Form extent: (self class fieldExtendFor: self playingField) depth: Display depth).
	self fieldForm fillColor: Color white.
	self setExtent.
	self setupMorphs.
	self playingField render: self fieldForm.
	ActiveHand newKeyboardFocus: self. "activates the keystroke detection methods"
	Transcript show: 'Morph ready'; cr. 
	
	self startStepping.! !

!PacGame methodsFor: 'keyboard' stamp: 'mw 6/29/2012 23:39'!
keyStroke: anEvent
	"Transcript show: 'KeyValue: '."
	"Transcript show: anEvent keyValue; cr. "
	"Event processing here"
	
	"player 1"
	(anEvent keyValue = 31) ifTrue: [
		"Transcript show: 'down'."
		"playingField move: 1 direction: 'down'."
		playingField pawn1 direction: Pawn directionDown.
	].
	(anEvent keyValue = 30) ifTrue: [
		"Transcript show: 'up'."
		"playingField move: 1 direction: 'up'."
		playingField pawn1 direction: Pawn directionUp.
	].
	(anEvent keyValue = 29) ifTrue: [
		"Transcript show: 'right'."
		"playingField move: 1 direction: 'right'."
		playingField pawn1 direction: Pawn directionRight..
	].
	(anEvent keyValue = 28) ifTrue: [
		"Transcript show: 'left'."
		"playingField move: 1 direction: 'left'."
		playingField pawn1 direction: Pawn directionLeft.
	].
	
	"player2 "
	(anEvent keyValue = 97) ifTrue: [
		playingField pawn2 direction: Pawn directionLeft.
	].
	(anEvent keyValue = 115) ifTrue: [
		playingField pawn2 direction: Pawn directionDown.
	].
	(anEvent keyValue = 100) ifTrue: [
		playingField pawn2 direction: Pawn directionRight.
	].
	(anEvent keyValue = 119) ifTrue: [
		playingField pawn2 direction: Pawn directionUp.
	].

	"self refresh"! !

!PacGame methodsFor: 'keyboard' stamp: 'tm 6/26/2012 01:13'!
keyboardFocusChange: aBoolean 
	aBoolean 
		ifTrue: [ Transcript show: 'Got Focus'; cr ]
		ifFalse: [ Transcript show: 'Lost Focus'; cr ]! !

!PacGame methodsFor: 'button' stamp: 'mw 6/9/2012 18:38'!
makeButton: label action: actionSymbol state: stateSymbol
	"generates a button"
	| btn labelMorph |
	btn := PluggableButtonMorph
			on: self
			getState: stateSymbol
			action: actionSymbol.
	labelMorph := StringMorph contents: label.
	labelMorph emphasis: 1.
	btn
		label: labelMorph;
		useRoundedCorners;
		hResizing: #spaceFill;
		onColor: Color veryLightGray offColor: Color lightGray;
		borderWidth: 2;
		borderColor: #raised.
	btn label color: Color darkGray.
	^btn! !

!PacGame methodsFor: 'drawing' stamp: 'mw 7/3/2012 23:40'!
makeControlPanelMorph
	| panel buttonHeight vertOffsetTop vertOffsetBtm |
	panel := RectangleMorph new borderWidth: 0;
			color: Color transparent;
			layoutPolicy: ProportionalLayout new.
	panel
		addMorph: (self makePointCounterGraph: 'player1')
		fullFrame: (LayoutFrame
			fractions: (0 @ 0 corner: 1 @ 0)
            	offsets: (4 @ 4 corner: -8 @ 44)).	
	panel
		addMorph: (self makePointCounterGraph: 'player2')
		fullFrame: (LayoutFrame
			fractions: (0 @ 0 corner: 1 @ 0)
            	offsets: (4 @ 54 corner: -8 @ 94)).	
	panel
		addMorph: (self makePointCounterGraph: 'player3')
		fullFrame: (LayoutFrame
			fractions: (0 @ 0 corner: 1 @ 0)
            	offsets: (4 @ 104 corner: -8 @ 144)).	
	panel
		addMorph: (self makePointCounterGraph: 'round')
		fullFrame: (LayoutFrame
			fractions: (0 @ 0 corner: 1 @ 0)
            	offsets: (4 @ 154 corner: -8 @ 194)).	
			
	buttonHeight := 26.
	vertOffsetTop := 10 + buttonHeight + 10 + buttonHeight.
	vertOffsetBtm := 10 + buttonHeight + 10.
	panel
		addMorph: self makeQuitGameButton
		fullFrame: (LayoutFrame
				fractions: (0 @ 1 corner: 1 @ 1)
				offsets: (
					(20 @ (vertOffsetTop negated))
						corner: (-20 @ (vertOffsetBtm negated)))).
						
	"vertOffsetTop := 10 + buttonHeight.
	vertOffsetBtm := 10.
	panel
		addMorph: self makeSomeButton
		fullFrame: (LayoutFrame
				fractions: (0 @ 1 corner: 1 @ 1)
				offsets: (
					(20 @ (vertOffsetTop negated))
						corner: (-20 @ (vertOffsetBtm negated)))).
	"
	^panel! !

!PacGame methodsFor: 'drawing' stamp: 'mw 6/9/2012 17:53'!
makeGameFieldMorph
	| boardMorph |
	boardMorph := SketchMorph withForm: self fieldForm.
	^boardMorph! !

!PacGame methodsFor: 'drawing' stamp: 'mw 6/29/2012 23:54'!
makePointCounterGraph: player
    | count |
    count := LedMorph new
                digits: 3;
                extent: 3 * 10 @ 15;
                setBalloonText: ''.
    count color: (Color r: 0.674 g: 0.674 b: 0.96).
    count name: player.
    ^ self wrapPanel: count label: player! !

!PacGame methodsFor: 'button' stamp: 'mw 6/9/2012 18:37'!
makeQuitGameButton
	^self makeButton: 'Quit' action: #quitGame state: nil! !

!PacGame methodsFor: 'drawing' stamp: 'mw 6/13/2012 23:02'!
panelWidth
	^200
! !

!PacGame methodsFor: 'accessing' stamp: 'mw 6/9/2012 17:26'!
playingField
	"Answer the value of playingField"

	^ playingField! !

!PacGame methodsFor: 'accessing' stamp: 'mw 6/9/2012 17:26'!
playingField: anObject
	"Set the value of playingField"

	playingField := anObject! !

!PacGame methodsFor: 'button' stamp: 'd 7/4/2012 22:08'!
quitGame
	"quits the game"
	Transcript show: 'quitting'; cr. 
	self colorProcess terminate.
	self stopStepping.
	self delete! !

!PacGame methodsFor: 'drawing' stamp: 'mw 6/30/2012 00:15'!
refresh
	"Transcript show: 'refreshing'; cr."
	self fieldForm changed.
	self updateCounters.
	self playingField render: self fieldForm.
	self changed. "This redraws the playing field without having to click on it again."! !

!PacGame methodsFor: 'accessing' stamp: 'mw 7/3/2012 23:40'!
rounds
	"Answer the value of rounds"
	rounds isNil ifTrue: [self rounds: 1].
	^ rounds! !

!PacGame methodsFor: 'accessing' stamp: 'mw 7/3/2012 23:39'!
rounds: anObject
	"Set the value of rounds"

	rounds := anObject! !

!PacGame methodsFor: 'accessing' stamp: 'mw 7/3/2012 23:38'!
running
	"Answer the value of running"
	running isNil ifTrue: [self running: true].
	^ running! !

!PacGame methodsFor: 'accessing' stamp: 'mw 7/3/2012 23:29'!
running: anObject
	"Set the value of running"

	running := anObject! !

!PacGame methodsFor: 'drawing' stamp: 'mw 6/9/2012 17:45'!
setExtent
	self extent: self calculateExtent! !

!PacGame methodsFor: 'drawing' stamp: 'mw 6/30/2012 00:04'!
setWindowColors
    self color: (Color
        r: 0.369
        g: 0.369
        b: 0.505).
    self fillWithRamp: self windowColorRamp oriented: 0.3@0.8! !

!PacGame methodsFor: 'initialize-release' stamp: 'mw 6/30/2012 00:04'!
setupMorphs
	self layoutPolicy: ProportionalLayout new.
	self setWindowColors.
	self
		addMorph: self makeGameFieldMorph
		fullFrame: (LayoutFrame
				fractions: (0 @ 0 corner: 1 @ 1)
				offsets: (0 @ 0 corner: self panelWidth negated @ 0)).
	self
		addMorph: self makeControlPanelMorph
		fullFrame: (LayoutFrame
				fractions: (1 @ 0 corner: 1 @ 1)
				offsets: (self panelWidth negated @ 0 corner: 0 @ 0)).! !

!PacGame methodsFor: 'gameloop' stamp: 'mw 7/4/2012 00:13'!
step
	"Is called at a regular intervall (defined by stepTime) after startStepping has been called until stopStepping is called."

	"playingField move: 1 direction: playingField pawn1 direction."
	self running 
		ifTrue: [
			
			((playingField pawn1 isDead) | (playingField pawn2 isDead) | (playingField pawn3 isDead)) ifTrue: [
				self running: false.
				Transcript cr; cr; show: 'Round over!!'; cr.
			] ifFalse: [			
				playingField pawn1 move: playingField pawn1 direction.
				playingField pawn2 move: playingField pawn2 direction.
				playingField pawn3 move: playingField pawn3 direction.
			]
		] ifFalse: [
			| oldField |
			oldField := self playingField.
			self playingField: PacGame createPlayingField.
			PacGame resetPlayers: playingField from: oldField.
			self rounds: (self rounds + 1).
			self running: true.
		].
	self refresh
	! !

!PacGame methodsFor: 'drawing' stamp: 'mw 7/3/2012 23:40'!
updateCounters
    | player1c |
    player1c := self findPlayerCounter: 'player1'.
    player1c notNil ifTrue: [
        player1c
                highlighted: true;
                value: self playingField pawn1 points.
            ].
player1c := self findPlayerCounter: 'player2'.
    player1c notNil ifTrue: [
        player1c
                highlighted: true;
                value: self playingField pawn2 points.
            ].
player1c := self findPlayerCounter: 'player3'.
    player1c notNil ifTrue: [
        player1c
                highlighted: true;
                value: self playingField pawn3 points.
            ].
player1c := self findPlayerCounter: 'round'.
    player1c notNil ifTrue: [
        player1c
                highlighted: true;
                value: self rounds.
            ]! !

!PacGame methodsFor: 'drawing' stamp: 'mw 6/30/2012 00:04'!
windowColorRamp
    ^ {0.0 -> (Color r: 0.3 g: 0.8 b: 0.9).
        1.0 -> (Color r: 0.2 g: 0.1 b: 0.7)}! !

!PacGame methodsFor: 'drawing' stamp: 'mw 6/29/2012 23:57'!
wrapPanel: aPanel label: aLabel 
    "wrap a panel in an alignmentMorph and put a label above it"
    | column strM |
    column := AlignmentMorph newColumn 
                wrapCentering: #topLeft;
                cellPositioning: #topLeft;
                hResizing: #spaceFill;
                vResizing: #shrinkWrap;
                borderWidth: 2;
                layoutInset: 5;
                color: Color transparent;
                useRoundedCorners;
                borderStyle: (BorderStyle complexAltInset width: 2).
    column addMorph: aPanel.
    strM := StringMorph contents: aLabel.
    strM color: Color veryVeryLightGray.
    column addMorph: strM.
    ^ column! !
